from aleksis.core.celery import app

from .commands import COMMANDS_BY_TASK_NAME, ImportCommand

TASKS = {}
for import_command in ImportCommand.__subclasses__():

    @app.task(name=import_command.task_name, bind=True)
    def _task(self, *args, **kwargs):
        import_command = COMMANDS_BY_TASK_NAME[self.name]
        import_command.run(*args, **kwargs)

    TASKS[import_command.task_name] = _task



import logging

from tqdm import tqdm

from aleksis.apps.chronos import models as chronos_models
from aleksis.apps.chronos.models import ValidityRange

from .... import models as mysql_models
from ..util import (
    TQDM_DEFAULTS,
    connect_untis_fields,
    date_to_untis_date,
    get_first_period,
    get_last_period,
    move_weekday_to_range,
    run_default_filter,
    untis_date_to_date,
)

logger = logging.getLogger(__name__)


def import_events(
    validity_range: ValidityRange,
    time_periods_ref,
    teachers_ref,
    classes_ref,
    rooms_ref,
):
    ref = {}

    # Get absences
    events = (
        run_default_filter(validity_range, mysql_models.Event.objects, filter_term=False)
        .filter(
            datefrom__lte=date_to_untis_date(validity_range.date_end),
            dateto__gte=date_to_untis_date(validity_range.date_start),
        )
        .order_by("event_id")
    )

    existing_events = []
    for event in tqdm(events, desc="Import events", **TQDM_DEFAULTS):
        import_ref = event.event_id

        logger.info("Import event {}".format(import_ref))

        # Build values
        comment = event.text

        date_from = untis_date_to_date(event.datefrom)
        date_to = untis_date_to_date(event.dateto)
        period_from = event.lessonfrom
        period_to = event.lessonto
        weekday_from = date_from.weekday()
        weekday_to = date_to.weekday()

        # Check min/max weekdays
        weekday_from = move_weekday_to_range(time_periods_ref, weekday_from)
        weekday_to = move_weekday_to_range(time_periods_ref, weekday_to)

        # Check min/max periods
        first_period = get_first_period(time_periods_ref, weekday_from)
        last_period = get_last_period(time_periods_ref, weekday_from)

        if period_from == 0:
            period_from = first_period
        if period_to == 0:
            period_to = last_period

        time_period_from = time_periods_ref[weekday_from][period_from]
        time_period_to = time_periods_ref[weekday_to][period_to]

        groups = []
        teachers = []
        rooms = []

        # Get groups, teachers and rooms
        raw_events = connect_untis_fields(event, "eventelement", 10)
        for raw_event in raw_events:
            el = raw_event.split("~")

            # Group
            if el[0] != "0" and el[0] != "":
                group = classes_ref[int(el[0])]
                groups.append(group)

            # Teacher
            if el[2] != "0" and el[2] != "":
                teacher = teachers_ref[int(el[2])]
                teachers.append(teacher)

            # Room
            if el[3] != "0" and el[3] != "":
                room = rooms_ref[int(el[3])]
                rooms.append(room)

        new_event, created = chronos_models.Event.objects.get_or_create(
            import_ref_untis=import_ref,
            defaults={
                "date_start": date_from,
                "date_end": date_to,
                "period_from": time_period_from,
                "period_to": time_period_to,
                "title": comment,
                "school_term": validity_range.school_term,
            },
        )

        if created:
            logger.info("  New event created")

        # Sync simple fields
        if (
            new_event.date_start != date_from
            or new_event.date_end != date_to
            or new_event.period_from != time_period_from
            or new_event.period_to != time_period_to
            or new_event.title != comment
            or new_event.school_term != validity_range.school_term
        ):
            new_event.date_start = date_from
            new_event.date_end = date_to
            new_event.period_from = time_period_from
            new_event.period_to = time_period_to
            new_event.title = comment
            new_event.school_term = validity_range.school_term
            new_event.save()
            logger.info("  Time range and title updated")

        # Sync m2m-fields
        new_event.groups.set(groups)
        new_event.teachers.set(teachers)
        new_event.rooms.set(rooms)

        existing_events.append(import_ref)
        ref[import_ref] = new_event

        # Delete all no longer existing events
        for e in chronos_models.Event.objects.within_dates(
            validity_range.date_start, validity_range.date_end
        ):
            if e.import_ref_untis and e.import_ref_untis not in existing_events:
                logger.info("Event {} deleted".format(e.id))
                e.delete()



from aleksis.core.settings import _settings

if _settings.get("untis.database.enabled"):
    DATABASES = {
        "untis": {
            "ENGINE": "django_prometheus.db.backends.mysql",
            "NAME": _settings.get("untis.database.name", "untis"),
            "USER": _settings.get("untis.database.user", "untis"),
            "PASSWORD": _settings.get("untis.database.password", None),
            "HOST": _settings.get("untis.database.host", "127.0.0.1"),
            "PORT": _settings.get("untis.database.port", 3306),
            "CONN_MAX_AGE": _settings.get("untis.database.conn_max_age", 0),
            "OPTIONS": _settings.get("untis.database.options", {}),
        }
    }



from rules import add_perm

from aleksis.core.util.predicates import has_global_perm, has_person

assign_subjects_to_groups_predicate = has_person & has_global_perm(
    "untis.assign_subjects_to_groups"
)
add_perm("untis.assign_subjects_to_groups_rule", assign_subjects_to_groups_predicate)



import logging
from typing import Dict

from tqdm import tqdm

from aleksis.apps.chronos import models as chronos_models
from aleksis.apps.chronos.models import ValidityRange

from .... import models as mysql_models
from ..util import TQDM_DEFAULTS, run_default_filter, untis_date_to_date

logger = logging.getLogger(__name__)


def import_holidays(validity_range: ValidityRange) -> Dict[int, chronos_models.Holiday]:
    """Import holidays."""
    ref = {}

    # Get holidays
    holidays = run_default_filter(validity_range, mysql_models.Holiday.objects, filter_term=False)

    for holiday in tqdm(holidays, desc="Import holidays", **TQDM_DEFAULTS):
        import_ref = holiday.holiday_id

        # Check if needed data are provided
        if not holiday.name:
            raise RuntimeError(
                "Holiday ID {}: Cannot import holiday without short name.".format(import_ref)
            )

        title = holiday.name[:50]
        comments = holiday.longname

        date_start = untis_date_to_date(holiday.datefrom)
        date_end = untis_date_to_date(holiday.dateto)

        logger.info("Import holiday {} …".format(title))

        # Get or create holiday
        new_holiday, created = chronos_models.Holiday.objects.get_or_create(
            import_ref_untis=import_ref,
            defaults={
                "title": title,
                "comments": comments,
                "date_start": date_start,
                "date_end": date_end,
            },
        )

        if created:
            logger.info("  New holiday created")

        if (
            new_holiday.title != title
            or new_holiday.comments != comments
            or new_holiday.date_start != date_start
            or new_holiday.date_end != date_end
        ):
            new_holiday.title = title
            new_holiday.comments = comments
            new_holiday.date_start = date_start
            new_holiday.date_end = date_end
            new_holiday.save()
            logger.info("  Holiday updated")

        ref[import_ref] = new_holiday

    return ref
